@@ -85,54 +85,82 @@ def separateWaveformTypes(file, headertype):
 	stoploc = len(file)
 	#setup the output containers
 	singles = []
+	singlesChecksums = []
 	coincidences = []
+	coincidencesChecksums = []
 	pulsers = []
+	pulsersChecksums = []
 	baselines = []
+	baselinesChecksums = []
 	cosmics = []
+	cosmicsChecksums = []
 	while curloc < stoploc:
+		#first grab the length of the waveform
 		length = file[curloc] - 13
+		#step past that
 		curloc += 1
+		#now this is the header itself
 		header = file[curloc:curloc+headerlength].view(headertype)[0]
-		if header['event type'] == 0:
+		#check what type of waveform it is and depending on that send it somewhere else
+		#at this stage the headers and waveforms have not been modified, just sent somewhere
+		eventType = header['event type']
+		tempWaveform = bf.doAnd(file[curloc+headerlength:curloc+length+headerlength])
+		checksum = bf.Fletcher32(tempWaveform)
+		if eventType == 0:
 			singles.append(file[curloc:curloc+length+headerlength])
+			singlesChecksums.append(checksum)
 		elif header['event type'] == 1:
 			coincidences.append(file[curloc:curloc+length+headerlength])
+			coincidencesChecksums.append(checksum)
 		elif header['event type'] == 2:
 			pulsers.append(file[curloc:curloc+length+headerlength])
+			pulsersChecksums.append(checksum)
 		elif header['event type'] == 3:
 			baselines.append(file[curloc:curloc+length+headerlength])
+			baselinesChecksums.append(checksum)
 		elif header['event type'] == 4:
 			cosmics.append(file[curloc:curloc+length+headerlength])
+			cosmicsChecksums.append(checksum)
 		curloc += length + headerlength
-	return singles, coincidences, pulsers, baselines, cosmics
+	return singles, singlesChecksums, coincidences, coincidencesChecksums, pulsers, pulsersChecksums, baselines, baselinesChecksums, cosmics, cosmicsChecksums
 
 def parseFile(filename, headertype):
 	file = np.fromfile(filename, dtype=np.uint16)
-	singles, coincidences, pulsers, baselines, cosmics = separateWaveformTypes(file, headertype)
+	singles, singlesChecksums, coincidences, coincidencesChecksums, pulsers, pulsersChecksums, baselines, baselinesChecksums, cosmics, cosmcsChecksums = separateWaveformTypes(file, headertype)
 	if len(singles) > 0:
 		singles = np.asarray(singles)
+		singlesChecksums = np.asarray(singlesChecksums).astype(np.uint32)
 	else:
 		singles = None
+		singlesChecksums = None
 	if len(coincidences) > 0:
 		coincidences = np.asarray(coincidences)
+		coincidencesChecksums = np.asarray(coincidencesChecksums).astype(np.uint32)
 	else:
 		coincidences = None
+		coincidencesChecksums = None
 	if len(pulsers) > 0:
 		pulsers = np.asarray(pulsers)
+		pulsersChecksums = np.asarray(pulsersChecksums).astype(np.uint32)
 	else:
 		pulsers = None
+		pulsersChecksums = None
 	if len(baselines) > 0:
 		baselines = np.asarray(baselines)
+		baselinesChecksums = np.asarray(baselinesChecksums).astype(np.uint32)
 	else:
 		baselines = None
+		baselinesChecksums = None
 	if len(cosmics) > 0:
 		cosmics = np.asarray(cosmics)
+		cosmicsChecksums = np.asarray(cosmicsChecksums).astype(np.uint32)
 	else:
 		cosmics = None
+		cosmicsChecksums = None
 	del file
-	return singles, coincidences, pulsers, baselines, cosmics
+	return singles, singlesChecksums, coincidences, coincidencesChecksums, pulsers, pulsersChecksums, baselines, baselinesChecksums, cosmics, cosmicsChecksums
 
-def prepareDataset(init):
+def prepareDataset(init, checksums):
 	'''
 	This function needs to prepare the dataset for writing to HDF5
 	Need to sort by the event id information first
@@ -140,13 +168,20 @@ def prepareDataset(init):
 	Then do the bitshift operation on the waveform data
 	'''
 	#first do the sorting of the dataset
-	indices = np.argsort(init['header']['eventid']) 
-	dtype = init['header'][indices].dtype
-	headers = np.copy(init['header'][indices])
+	indices = np.argsort(init['header']['eventid'])
+	newHeaderType = []
+	for element in init['header'].dtype.descr:
+		newHeaderType.append(element)
+	newHeaderType.append(('checksum', 'u4'))
+	newHeaders = np.zeros(len(indices), dtype=np.dtype(newHeaderType))
+	for name in init['header'].dtype.names:
+		newHeaders[name] = np.copy(init['header'][name][indices])
+	newHeaders['checksum'] = np.copy(checksums[indices])
 	waveforms = np.copy(bf.doAnd(init['wave'][indices]))
 	del init
+	del checksums
 	del indices
-	return headers, waveforms
+	return newHeaders, waveforms
 
 def parseWaveformFiles(directory, run, subrun, headertype):
 	'''
@@ -155,74 +190,89 @@ def parseWaveformFiles(directory, run, subrun, headertype):
 	'''
 	files = glob.glob(directory+'Run'+str(run)+'_'+str(subrun)+'*.wave')
 	singles = []
+	singlesChecksums = []
 	coincidences = []
+	coincidencesChecksums = []
 	pulsers = []
+	pulsersChecksums = []
 	baselines = []
+	baselinesChecksums = []
 	cosmics = []
+	cosmicsChecksums = []
 	for file in files:
-		s, c, p, b, cos = parseFile(file, headertype)
+		s, sChecks, c, cChecks, p, pChecks, b, bChecks, cos, cosChecks = parseFile(file, headertype)
 		if s is not None:
 			singles.append(s)
+			singlesChecksums.append(sChecks)
 		if c is not None:
 			coincidences.append(c)
+			coincidencesChecksums.append(cChecks)
 		if p is not None:
 			pulsers.append(p)
+			pulsersChecksums.append(pChecks)
 		if b is not None:
 			baselines.append(b)
+			baselinesChecksums.append(bChecks)
 		if cos is not None:
 			cosmics.append(cos)
+			cosmicsChecksums.append(cosChecks)
 	#make singular arrays for each of these
 	if len(singles) > 0:
 		singles = np.concatenate(singles)
+		singlesChecksums = np.concatenate(singlesChecksums)
 		if singles.ndim == 2:
 			wavelength = singles.shape[1] - 13
 			dtype = ff.headWaveType(wavelength, 'Nab', 4)
 			singles = singles.view(dtype)[:,0]
-			singles = prepareDataset(singles)
+			singles = prepareDataset(singles, singlesChecksums)
 		else:
 			singles = None
 	else:
 		singles = None
 	if len(coincidences) > 0:
 		coincidences = np.concatenate(coincidences)
+		coincidencesChecksums = np.concatenate(coincidencesChecksums)
 		if coincidences.ndim == 2:
 			wavelength = coincidences.shape[1] - 13
 			dtype = ff.headWaveType(wavelength, 'Nab', 4)
 			coincidences = coincidences.view(dtype)[:,0]
-			coincidences = prepareDataset(coincidences)
+			coincidences = prepareDataset(coincidences, coincidencesChecksums)
 		else:
 			coincidences = None
 	else:
 		coincidences = None
 	if len(pulsers) > 0:
 		pulsers = np.concatenate(pulsers)
+		pulsersChecksums = np.concatenate(pulsersChecksums)
 		if pulsers.ndim == 2:
 			wavelength = pulsers.shape[1] - 13
 			dtype = ff.headWaveType(wavelength, 'Nab', 4)
 			pulsers = pulsers.view(dtype)[:,0]
-			pulsers = prepareDataset(pulsers)
+			pulsers = prepareDataset(pulsers, pulsersChecksums)
 		else:
 			pulsers = None
 	else:
 		pulsers = None
 	if len(baselines) > 0:
 		baselines = np.concatenate(baselines)
+		baselinesChecksums = np.concatenate(baselinesChecksums)
 		if baselines.ndim == 2:
 			wavelength = baselines.shape[1] - 13
 			dtype = ff.headWaveType(wavelength, 'Nab', 4)
 			baselines = baselines.view(dtype)[:,0]
-			baselines = prepareDataset(baselines)
+			baselines = prepareDataset(baselines, baselinesChecksums)
 		else:
 			baselines = None
 	else:
 		baselines = None
 	if len(cosmics) > 0:
 		cosmics = np.concatenate(cosmics)
+		cosmicsChecksums = np.concatenate(cosmicsChecksums)
 		if cosmics.ndim == 2:
 			wavelength = cosmics.shape[1] - 13
 			dtype = ff.headWaveType(wavelength, 'Nab', 4)
 			cosmics = cosmics.view(dtype)[:,0]
-			cosmics = prepareDataset(cosmics)
+			cosmics = prepareDataset(cosmics, cosmicsChecksums)
 		else:
 			cosmics = None
 	else:
